/****************************************************************************
 * (C) Nicolas GEORGE 1999                                                  *
 * Ce programme fait partie du projet xbackground.                          *
 * Il est distribu sous les termes de la GNU-GPL, dont un exemplaire doit  *
 * tre joint (si ce n'est pas le cas, vous le trouverez sur www.fsf.org).  *
 * Je ne fournis aucune garantie sur ce programme.                          *
 ****************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "xbackground.h"

static int minr=32768,minv=32768,minb=32768;
static int maxr=65535,maxv=65535,maxb=65535;
static int lt=128,ht=96;

static int lire_chaine(char *c,char *v)
{
  if((!strcmp(c,"dark"))||(!strcmp(c,"light"))){
    XColor co;

    if(!XParseColor(disp,colormap,v,&co)){
      fprintf(stderr,"Invalid color: %s\n",v);
      return(-1);
    }
    if(!strcmp(c,"dark")){
      minr=co.red;
      minv=co.green;
      minb=co.blue;
    }else{
      maxr=co.red;
      maxv=co.green;
      maxb=co.blue;
    }
    return(1);
  }else if(!strcmp(c,"width")){
    int i;

    lt=atoi(v);
    for(i=0,lt--;lt>0;i++,lt>>=1);
    lt=1<<i;
    if(lt<8) lt=8;
    if(lt>1024) lt=1024;
    return(1);
  }else if(!strcmp(c,"height")){
    ht=atoi(v);
    if(ht<8) ht=8;
    if(ht>1024) ht=1024;
    return(1);
  }
  return(0);
}

static int initialiser(int *l,int *h)
{
  int i,r,v,b;

  if(*l>lt) *l=lt;
  if(*h>ht) *h=ht;

  for(i=0;i<nb_col;i++){
    r=minr+(i*(maxr-minr))/(nb_col-1);
    r=r<0?0:r>65535?65535:r;
    v=minv+(i*(maxv-minv))/(nb_col-1);
    v=v<0?0:v>65535?65535:v;
    b=minb+(i*(maxb-minb))/(nb_col-1);
    b=b<0?0:b>65535?65535:b;
    pal[i]=((long)(r/257))|(((long)(v/257))<<8)|(((long)(b/257))<<16);
  }
  return(0);
}

#define ramp(i) (rand()%(((i)<<1)+1)-((i)<<0))

static void creer_cascade(int ls,int hs,XImage *img)
{
  int i,x,y;
  short *t;

  t=malloc(sizeof(short)*lt);
  t[0]=0;
  for(i=ls>>1;i>0;i>>=1)
    for(x=0;x<ls;x+=i<<1)
      t[x+i]=((t[x]+t[(x+(i<<1))&(ls-1)])>>1)+ramp(i);

  for(y=0;y<hs;y++)
    for(x=0;x<ls;x++){
      i=((int)t[x]+y+65536)%ht;
      i=256*i/ht;
      if(i<0) i=0;
      if(i>255) i=255;
      XPutPixel(img,x,y,pal[i*(nb_col-1)/255]);
    }
  free(t);
}

moteur_de_fond moteur_cascade={
  "cascade",
  lire_chaine,
  initialiser,
  creer_cascade
};
