/****************************************************************************
 * (C) Nicolas GEORGE 1999                                                  *
 * Ce programme fait partie du projet xbackground.                          *
 * Il est distribu sous les termes de la GNU-GPL, dont un exemplaire doit  *
 * tre joint (si ce n'est pas le cas, vous le trouverez sur www.fsf.org).  *
 * Je ne fournis aucune garantie sur ce programme.                          *
 ****************************************************************************/

#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "xbackground.h"

#ifdef __cplusplus
#define v_class c_class
#else /* __cplusplus */
#define v_class class
#endif /* __cplusplus */

extern moteur_de_fond moteur_plasma;
extern moteur_de_fond moteur_cascade;
extern moteur_de_fond moteur_mandelbrot;

moteur_de_fond *moteurs[]={
  &moteur_plasma,
  &moteur_cascade,
  &moteur_mandelbrot,
  NULL
};
moteur_de_fond *moteur=&moteur_plasma;

unsigned long *pal;
int nb_col=256;

void ouvrir_serveux_X(void)
{
  int screen;
  XWindowAttributes a;
  XGCValues gv;

  disp=XOpenDisplay(NULL);
  if(disp==NULL){
    fprintf(stderr,"Connexion refused by X server\n");
    abort();
  }
  screen=DefaultScreen(disp);
  depth=DefaultDepth(disp,screen);
  colormap=DefaultColormap(disp,screen);

  win=RootWindow(disp,screen);

  gv.line_width=1;
  gv.function=GXcopy;
  gc=XCreateGC(disp,win,GCFunction|GCLineWidth,&gv);

  XGetWindowAttributes(disp,win,&a);
  ls=a.width; hs=a.height;
  vis=a.visual;

  XSetCloseDownMode(disp,RetainPermanent);
}

void calculer_palette(void)
{
  int i;
  XColor c;
  unsigned long p;

  /* Je devrais allouer de manire moins linaire (du style les extrmes,
     et progressivement les milieux), mais pas tout de suite. */
  for(i=0;i<nb_col;i++){
    c.red  =257*(pal[i]&255);
    c.green=257*((pal[i]>>8)&255);
    c.blue =257*((pal[i]>>16)&255);
    c.flags=DoRed|DoGreen|DoBlue;
    if(XAllocColor(disp,colormap,&c))
      p=c.pixel;
    pal[i]=p;
  }
}

int main(void)
{
  setpriority(PRIO_PROCESS,0,19);

  ouvrir_serveux_X();

  srand(time(NULL));

  lire_parametres();
  pal=malloc(nb_col*sizeof(unsigned long));

  switch(vis->v_class){
    unsigned long i;

    case PseudoColor:
      break;
    case TrueColor:
    case DirectColor:
      i=vis->red_mask;
      for(nd_r=0;(i&1)==0;i>>=1) nd_r++;
      for(nb_r=16;i&1;i>>=1) nb_r--;

      i=vis->green_mask;
      for(nd_v=0;(i&1)==0;i>>=1) nd_v++;
      for(nb_v=16;i&1;i>>=1) nb_v--;

      i=vis->blue_mask;
      for(nd_b=0;(i&1)==0;i>>=1) nd_b++;
      for(nb_b=16;i&1;i>>=1) nb_b--;
      break;
    default:
      fprintf(stderr,"Visual not suported\n");
      abort();
  }

  if(moteur->initialiser(&ls,&hs))
    return(1);

  calculer_palette();

  pix=XCreatePixmap(disp,win,ls,hs,depth);
  XSetWindowBackgroundPixmap(disp,win,pix);
  img=XCreateImage(disp,vis,depth,ZPixmap,0,NULL,ls,hs,32,0);
  img->data=malloc(img->bytes_per_line*img->height);

  moteur->calculer(ls,hs,img);

  XPutImage(disp,pix,gc,img,0,0,0,0,ls,hs);
  XClearWindow(disp,win);
  XSync(disp,False);

  XFreeGC(disp,gc);
  XFreePixmap(disp,pix);
  XDestroyImage(img);

  return(0);
}
